import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;

import java.net.InetAddress;
import java.net.UnknownHostException;

import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JOptionPane;

import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupID;

import net.jxta.discovery.DiscoveryService;
import net.jxta.discovery.DiscoveryEvent;
import net.jxta.discovery.DiscoveryListener;

import net.jxta.protocol.DiscoveryResponseMsg;
import net.jxta.protocol.PipeAdvertisement;

import net.jxta.document.AdvertisementFactory;
import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredTextDocument;
import net.jxta.id.IDFactory;

import net.jxta.pipe.PipeService;
import net.jxta.pipe.InputPipe;
import net.jxta.pipe.OutputPipe;
import net.jxta.pipe.PipeMsgEvent;
import net.jxta.pipe.PipeMsgListener;
import net.jxta.pipe.OutputPipeListener;

import net.jxta.endpoint.Message;

import net.jxta.platform.Application;

/**
 *
 * @author  BGoetzmann
 * @version 
 */
public class NetSendApp extends JFrame implements Application,
    DiscoveryListener, PipeMsgListener, OutputPipeListener {
    
    /**
     * L'ID de module de classe pour l'application NetSend.
     */
    public static final String refModuleClassID = 
        "urn:jxta:uuid-E340D55F97E141C9B46FB2B108D8C2B705";
    
    /**
     * L'ID de spcification fde module pour l'application NetSend.
     */
    public static final String refModuleSpecID = 
        "urn:jxta:uuid-E340D55F97E141C9B46FB2B108D8C2B7"
        + "581B0312E66046B6BB96BF6A2EC5F27906";              
    
    /**
     * Le groupe de pair auquel l'application appartient.
     */
    private PeerGroup peerGroup = null;
    
    /**
     * Le service Discovery.
     */
    private DiscoveryService discovery = null;
    
    /**
     * Le service Pipe.
     */    
    private PipeService pipe = null;
    
    /**
     * Nom de l'hte local.
     */     
    private String localhost;
    
    /**
     * Texte du message  envoyer.
     */     
    private String message;
    
    /**
     * L'ensemble des ID des pipes, index par le nom du pipe
     */
    private Hashtable pipeIDs = new Hashtable();
    
    private static final String NETSEND = "NETSEND:";
    
    /**
     * L'objet InputPipe utilis pour recevoir des messages.
     */    
    private InputPipe inputPipe = null;
    
    /**
     * L'objet OutputPipe  utiliser pour envoyer un message au pair distant.
     */
    private OutputPipe outputPipe = null;    
    
    private JTextField messageText = new JTextField(20);
    
   
    public NetSendApp() {
        super("NetSend");
    }
    
    public void initializeUserInterface()
    {
        JButton sendButton = new JButton("Envoyer");

        Container pane = getContentPane();
        pane.setLayout(new FlowLayout());
        pane.add(messageText);
        pane.add(sendButton);
        
        getRootPane().setDefaultButton(sendButton);

        // Mise en place des couteurs pour les boutons.
        sendButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e)
                {    
                    // Rcuprer le nom de la machine destinataire
                    String msg = messageText.getText();
                    int pos = msg.indexOf(' ');
                    String name = null;
                    if (pos != -1) 
                        name = msg.substring(0, pos).toUpperCase();
                    if (name == null)
                        return;    
                    
                    // Sauvegarder le texte du message
                    message = msg.substring(pos + 1);
                    
                    try {
                        createOutputPipe(NETSEND + name);
                    }
                    catch (IOException ex) {
                        JOptionPane.showMessageDialog(null, "Impossible d'envoyer le message", "Erreur", JOptionPane.WARNING_MESSAGE);
                    }
                }
            }
        );               
        
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent winEvent) {
                    hide();
                    
                    // Arrter la plateforme JXTA.
                    System.exit(0);
                }
            });        
        
        pack();
    }

    public void stopApp() {
    }
    
    public int startApp(java.lang.String[] str) {
                              
        discovery = peerGroup.getDiscoveryService();
        
        try {
            discovery.flushAdvertisements(null, DiscoveryService.ADV);
        } 
        catch (IOException e) {
        }
        
        // S'ajouter soi-mme comme couteur
        discovery.addDiscoveryListener(this);
        
        // Envoyer une requte distante pour rechercher les annonces de Pipe.
        getRemotePipeAdvertisements("*");
        
        pipe = peerGroup.getPipeService();        
        
        try {
            localhost = InetAddress.getLocalHost().getHostName().toUpperCase();
        }
        catch (UnknownHostException e) {
        }
        
        try {
            PipeAdvertisement pipeAdv = getPipeAdvertisement("PipeAdv.xml");
                        
            publishPipeAdvertisement(pipeAdv);
        
            // Crer un objet InputPipe en utilisant l'annonce           
            inputPipe = pipe.createInputPipe(pipeAdv, this);                        
        }
        catch (IOException e) {
            System.out.println("Erreur : " + e);
        }
        
        // Afficher l'interface utilisateur
        initializeUserInterface();                
        setVisible(true);
        
        return 0;
    }
    
    /**
     * Renvoie l'annonce du Pipe cr et initialis depuis le fichier pass en
     * en argument si celui-ci existe, ou bien une nouvelle annonce de Pipe est
     * cre et sauvegarde dans le fichier.
     *        
     * @param   fileName le nom du fichier depuis on essaie de lire l'annonce
     *  du Pipe.
     * @return l'annonce du Pipe.
     */         
    private PipeAdvertisement getPipeAdvertisement(String filename) throws IOException {
        PipeAdvertisement pipeAdv = null;
        
        try {
            // Lecture du fichier de l'annonce.
            FileInputStream file = new FileInputStream(filename);
            MimeMediaType asMimeType = new MimeMediaType("text/xml");

            // Charger l'annonce.
            pipeAdv = (PipeAdvertisement) AdvertisementFactory.newAdvertisement(asMimeType, file);            
        }
        catch (FileNotFoundException e) {                   
            
            // Il nous faut crer le fichier
            pipeAdv = (PipeAdvertisement)AdvertisementFactory.newAdvertisement(PipeAdvertisement.getAdvertisementType());
            pipeAdv.setPipeID(IDFactory.newPipeID(peerGroup.getPeerGroupID()));                      
            
            // Nommer le Pipe
            pipeAdv.setName(NETSEND + localhost);                
                
            writePipeAdv(pipeAdv, filename);
        }
        
        return pipeAdv;
    }
    
    /**
     * Ecrit l'annonce de Pipe pass en argument dans un fichier avec le nom
     * spcifi.
     *
     * @param   pipeAdv l'annonce du Pipe  crire dans le fichier.
     * @param   fileName le nom du fichier dans lequel crire.
     */    
    private void writePipeAdv(PipeAdvertisement pipeAdv, String fileName)
    {        
        // Crer une version XML de l'annonce du Pipe
        try
        {
            FileWriter file = new FileWriter(fileName);
            MimeMediaType mimeType = new MimeMediaType("text/xml");
            StructuredTextDocument document = (StructuredTextDocument) pipeAdv.getDocument(mimeType);
            
            // Ecrire le document XML dans le fichier
            document.sendToWriter(file);
            file.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }    
    
    /**
     * Envoyer une requte distante pour rechercher les annonces de Pipe ayant
     * l'lment Name commenant par "NETSEND:".
     */
    public void getRemotePipeAdvertisements(String name) {
        discovery.getRemoteAdvertisements(null, DiscoveryService.ADV, "Name",
            NETSEND + name, Integer.MAX_VALUE, null);             
    }
    
    public void init(net.jxta.peergroup.PeerGroup peerGroup, net.jxta.id.ID iD, net.jxta.document.Advertisement advertisement) throws net.jxta.exception.PeerGroupException {
                
        this.peerGroup = peerGroup;                
    }
    
    /**
     * Envoyer une requte pour trouver l'annonce de Pipe ayant le nom pass
     * en paramtre.
     *
     * @param   name nom de l'annonce de Pipe.
     */
    public PipeAdvertisement findPipe(String name) {       
        
        PipeAdvertisement pipeAdv = null;              

        // Faire une recherche locale pour retrouver l'annonce du Pipe nomm
        // name et ayant t dcouverte.        
        try {
            Enumeration enum = discovery.getLocalAdvertisements(
                DiscoveryService.ADV, "Name", name);
                
            String pipeID = (String) pipeIDs.get(name);            
            
            if (pipeID != null) {
                while (enum.hasMoreElements()) {         
                    PipeAdvertisement padv =
                        (PipeAdvertisement) enum.nextElement();                               
                    
                    if ( padv.getName().equals(name) && 
                            padv.getPipeID().toString().equals(pipeID) ) {                                  
                                
                        pipeAdv = padv;
                        break;
                    }
                }
            }
            else
                JOptionPane.showMessageDialog(null,
                    "Impossible de trouver " + name, "Erreur", JOptionPane.WARNING_MESSAGE);
            
        }
        catch (IOException e) {
            System.out.println("Erreur dans findPipe : " + e);
        }        
        
        return pipeAdv;
    }
    
    private void publishPipeAdvertisement(PipeAdvertisement pipeAdv) throws IOException {
        
        // Publier l'annonce localement
        discovery.publish(pipeAdv, DiscoveryService.ADV);
        
        // Publier l'annonce de manire distante
        discovery.remotePublish(pipeAdv, DiscoveryService.ADV);       
    }           
    
    public void pipeMsgEvent(net.jxta.pipe.PipeMsgEvent pipeMsgEvent) {        
        
        // Extraire le message
        Message message = pipeMsgEvent.getMessage();

        // Afficher le texte du message.      
        JOptionPane.showMessageDialog(null, "Message de " +
            message.getString("PeerName") + " :\n\n" +
            message.getString("MessageText"), "NetSend", JOptionPane.INFORMATION_MESSAGE);
        
    }
    
    public void createOutputPipe(String name) throws IOException {        
        
        PipeAdvertisement pipeAdv = findPipe(name);
        if (pipeAdv != null)             
            pipe.createOutputPipe(pipeAdv, this);  
        else
            JOptionPane.showMessageDialog(null, "Impossible d'envoyer le message", "Erreur", JOptionPane.WARNING_MESSAGE);            
    }    
    
    public void outputPipeEvent(net.jxta.pipe.OutputPipeEvent outputPipeEvent) {       
        
        outputPipe = outputPipeEvent.getOutputPipe();
        
        // Envoyer le message.
        sendMessage(message); 
                
        // Effacer le texte.
        messageText.setText("");
    }    
    
    
    /**    
     * Envoie un message au pair distant en utilisant l'objet outputPipe.
     *
     * @param   messageString le message  envoyer au pair distant.
     */
    private void sendMessage(String messageString) {
        
        Message message = pipe.createMessage();
        
        message.setString("PeerName", peerGroup.getPeerName());
        message.setString("MessageText", messageString);

        if (null != outputPipe)
        {
            try
            {
                // Envoyer le message
                outputPipe.send(message);
            }
            catch (IOException e)
            {
                JOptionPane.showMessageDialog(null, e.toString(), "Erreur", JOptionPane.WARNING_MESSAGE);
            }
        }
        else
        {
            JOptionPane.showMessageDialog(null, "L'objet OutputPipe est null !", "Erreur", JOptionPane.WARNING_MESSAGE);
        }
    }
        
    /** 
     * Gre la notification de messages de rponse du service Discovery,
     * dtermine si la rponse contient une annonce de Pipe.
     *
     * @param   discoveryEvent l'objet contenant la rponse du service
     * Discovery.
     */    
    public void discoveryEvent(net.jxta.discovery.DiscoveryEvent discoveryEvent) {
        DiscoveryResponseMsg response = discoveryEvent.getResponse();
        
        // Extrait l'annonce de Pipe de la rponse.
        Enumeration responses = response.getResponses();
        while (responses.hasMoreElements())
        {
            String responseElement = (String) responses.nextElement();
                       
            // Vrifier qu'il y a bien une rponse.
            if (null != responseElement)
            {
                // Parcourir l'annonce.
                try
                {
                    ByteArrayInputStream stream = 
                        new ByteArrayInputStream(responseElement.getBytes());
                                        
                    PipeAdvertisement pipeAdv = (PipeAdvertisement) 
                        AdvertisementFactory.newAdvertisement(
                            new MimeMediaType("text/xml"), stream);                   
                    
                    System.out.println("\ndiscoveryEvent");
                    System.out.println("name : " + pipeAdv.getName() +
                        "\npipeID : " + pipeAdv.getPipeID());
                    
                    pipeIDs.put(pipeAdv.getName(), pipeAdv.getPipeID().toString());                    
                }
                catch (IOException e)
                {
                    // Ignorer l'erreur.
                    System.out.println("Erreur dans discoveryEvent : " + e);
                }
            }
            else
            {
                System.out.println("La rponse d'annonces vaut null !");
            }
        }
        
    }
    
}
